@@ -36,7 +36,7 @@ class BaseConnection(object):
 
     @abc.abstractmethod
     def get_goal_list(self, context, filters=None, limit=None,
-                      marker=None, sort_key=None, sort_dir=None):
+                      marker=None, sort_key=None, sort_dir=None, eager=False):
         """Get specific columns for matching goals.
 
         Return a list of the specified columns for all goals that
@@ -44,13 +44,13 @@ class BaseConnection(object):
 
         :param context: The security context
         :param filters: Filters to apply. Defaults to None.
-
         :param limit: Maximum number of goals to return.
         :param marker: the last item of the previous page; we return the next
                        result set.
         :param sort_key: Attribute by which results should be sorted.
         :param sort_dir: direction in which results should be sorted.
                          (asc, desc)
+        :param eager: If True, also loads One-to-X data (Default: False)
         :returns: A list of tuples of the specified columns.
         """
 
@@ -73,31 +73,34 @@ class BaseConnection(object):
         """
 
     @abc.abstractmethod
-    def get_goal_by_id(self, context, goal_id):
+    def get_goal_by_id(self, context, goal_id, eager=False):
         """Return a goal given its ID.
 
         :param context: The security context
         :param goal_id: The ID of a goal
+        :param eager: If True, also loads One-to-X data (Default: False)
         :returns: A goal
         :raises: :py:class:`~.GoalNotFound`
         """
 
     @abc.abstractmethod
-    def get_goal_by_uuid(self, context, goal_uuid):
+    def get_goal_by_uuid(self, context, goal_uuid, eager=False):
         """Return a goal given its UUID.
 
         :param context: The security context
         :param goal_uuid: The UUID of a goal
+        :param eager: If True, also loads One-to-X data (Default: False)
         :returns: A goal
         :raises: :py:class:`~.GoalNotFound`
         """
 
     @abc.abstractmethod
-    def get_goal_by_name(self, context, goal_name):
+    def get_goal_by_name(self, context, goal_name, eager=False):
         """Return a goal given its name.
 
         :param context: The security context
         :param goal_name: The name of a goal
+        :param eager: If True, also loads One-to-X data (Default: False)
         :returns: A goal
         :raises: :py:class:`~.GoalNotFound`
         """
@@ -130,17 +133,23 @@ class BaseConnection(object):
         :raises: :py:class:`~.Invalid`
         """
 
+    def soft_delete_goal(self, goal_id):
+        """Soft delete a goal.
+
+        :param goal_id: The id or uuid of a goal.
+        :raises: :py:class:`~.GoalNotFound`
+        """
+
     @abc.abstractmethod
     def get_strategy_list(self, context, filters=None, limit=None,
-                          marker=None, sort_key=None, sort_dir=None):
+                          marker=None, sort_key=None, sort_dir=None,
+                          eager=True):
         """Get specific columns for matching strategies.
 
         Return a list of the specified columns for all strategies that
         match the specified filters.
 
         :param context: The security context
-        :param columns: List of column names to return.
-                        Defaults to 'id' column when columns == None.
         :param filters: Filters to apply. Defaults to None.
 
         :param limit: Maximum number of strategies to return.
@@ -149,6 +158,7 @@ class BaseConnection(object):
         :param sort_key: Attribute by which results should be sorted.
         :param sort_dir: Direction in which results should be sorted.
                          (asc, desc)
+        :param eager: If True, also loads One-to-X data (Default: False)
         :returns: A list of tuples of the specified columns.
         """
 
@@ -173,31 +183,34 @@ class BaseConnection(object):
         """
 
     @abc.abstractmethod
-    def get_strategy_by_id(self, context, strategy_id):
+    def get_strategy_by_id(self, context, strategy_id, eager=False):
         """Return a strategy given its ID.
 
         :param context: The security context
         :param strategy_id: The ID of a strategy
+        :param eager: If True, also loads One-to-X data (Default: False)
         :returns: A strategy
         :raises: :py:class:`~.StrategyNotFound`
         """
 
     @abc.abstractmethod
-    def get_strategy_by_uuid(self, context, strategy_uuid):
+    def get_strategy_by_uuid(self, context, strategy_uuid, eager=False):
         """Return a strategy given its UUID.
 
         :param context: The security context
         :param strategy_uuid: The UUID of a strategy
+        :param eager: If True, also loads One-to-X data (Default: False)
         :returns: A strategy
         :raises: :py:class:`~.StrategyNotFound`
         """
 
     @abc.abstractmethod
-    def get_strategy_by_name(self, context, strategy_name):
+    def get_strategy_by_name(self, context, strategy_name, eager=False):
         """Return a strategy given its name.
 
         :param context: The security context
         :param strategy_name: The name of a strategy
+        :param eager: If True, also loads One-to-X data (Default: False)
         :returns: A strategy
         :raises: :py:class:`~.StrategyNotFound`
         """
@@ -220,26 +233,31 @@ class BaseConnection(object):
         :raises: :py:class:`~.Invalid`
         """
 
+    def soft_delete_strategy(self, strategy_id):
+        """Soft delete a strategy.
+
+        :param strategy_id: The id or uuid of a strategy.
+        :raises: :py:class:`~.StrategyNotFound`
+        """
+
     @abc.abstractmethod
-    def get_audit_template_list(self, context, columns=None, filters=None,
+    def get_audit_template_list(self, context, filters=None,
                                 limit=None, marker=None, sort_key=None,
-                                sort_dir=None):
+                                sort_dir=None, eager=False):
         """Get specific columns for matching audit templates.
 
         Return a list of the specified columns for all audit templates that
         match the specified filters.
 
         :param context: The security context
-        :param columns: List of column names to return.
-                        Defaults to 'id' column when columns == None.
         :param filters: Filters to apply. Defaults to None.
-
         :param limit: Maximum number of audit templates to return.
         :param marker: the last item of the previous page; we return the next
                        result set.
         :param sort_key: Attribute by which results should be sorted.
         :param sort_dir: direction in which results should be sorted.
                          (asc, desc)
+        :param eager: If True, also loads One-to-X data (Default: False)
         :returns: A list of tuples of the specified columns.
         """
 
@@ -256,46 +274,50 @@ class BaseConnection(object):
                          'uuid': utils.generate_uuid(),
                          'name': 'example',
                          'description': 'free text description'
-                         'host_aggregate': 'nova aggregate name or id'
                          'goal': 'DUMMY'
-                         'extra': {'automatic': True}
                         }
         :returns: An audit template.
         :raises: :py:class:`~.AuditTemplateAlreadyExists`
         """
 
     @abc.abstractmethod
-    def get_audit_template_by_id(self, context, audit_template_id):
+    def get_audit_template_by_id(self, context, audit_template_id,
+                                 eager=False):
         """Return an audit template.
 
         :param context: The security context
         :param audit_template_id: The id of an audit template.
+        :param eager: If True, also loads One-to-X data (Default: False)
         :returns: An audit template.
         :raises: :py:class:`~.AuditTemplateNotFound`
         """
 
     @abc.abstractmethod
-    def get_audit_template_by_uuid(self, context, audit_template_uuid):
+    def get_audit_template_by_uuid(self, context, audit_template_uuid,
+                                   eager=False):
         """Return an audit template.
 
         :param context: The security context
         :param audit_template_uuid: The uuid of an audit template.
+        :param eager: If True, also loads One-to-X data (Default: False)
         :returns: An audit template.
         :raises: :py:class:`~.AuditTemplateNotFound`
         """
 
-    def get_audit_template_by_name(self, context, audit_template_name):
+    def get_audit_template_by_name(self, context, audit_template_name,
+                                   eager=False):
         """Return an audit template.
 
         :param context: The security context
         :param audit_template_name: The name of an audit template.
+        :param eager: If True, also loads One-to-X data (Default: False)
         :returns: An audit template.
         :raises: :py:class:`~.AuditTemplateNotFound`
         """
 
     @abc.abstractmethod
     def destroy_audit_template(self, audit_template_id):
-        """Destroy an audit_template.
+        """Destroy an audit template.
 
         :param audit_template_id: The id or uuid of an audit template.
         :raises: :py:class:`~.AuditTemplateNotFound`
@@ -313,31 +335,29 @@ class BaseConnection(object):
 
     @abc.abstractmethod
     def soft_delete_audit_template(self, audit_template_id):
-        """Soft delete an audit_template.
+        """Soft delete an audit template.
 
         :param audit_template_id: The id or uuid of an audit template.
         :raises: :py:class:`~.AuditTemplateNotFound`
         """
 
     @abc.abstractmethod
-    def get_audit_list(self, context, columns=None, filters=None, limit=None,
-                       marker=None, sort_key=None, sort_dir=None):
+    def get_audit_list(self, context, filters=None, limit=None,
+                       marker=None, sort_key=None, sort_dir=None, eager=False):
         """Get specific columns for matching audits.
 
         Return a list of the specified columns for all audits that match the
         specified filters.
 
         :param context: The security context
-        :param columns: List of column names to return.
-                        Defaults to 'id' column when columns == None.
         :param filters: Filters to apply. Defaults to None.
-
         :param limit: Maximum number of audits to return.
         :param marker: the last item of the previous page; we return the next
                        result set.
         :param sort_key: Attribute by which results should be sorted.
         :param sort_dir: direction in which results should be sorted.
                          (asc, desc)
+        :param eager: If True, also loads One-to-X data (Default: False)
         :returns: A list of tuples of the specified columns.
         """
 
@@ -354,28 +374,29 @@ class BaseConnection(object):
                         {
                          'uuid': utils.generate_uuid(),
                          'type': 'ONESHOT',
-                         'deadline': None
                         }
         :returns: An audit.
         :raises: :py:class:`~.AuditAlreadyExists`
         """
 
     @abc.abstractmethod
-    def get_audit_by_id(self, context, audit_id):
+    def get_audit_by_id(self, context, audit_id, eager=False):
         """Return an audit.
 
         :param context: The security context
         :param audit_id: The id of an audit.
+        :param eager: If True, also loads One-to-X data (Default: False)
         :returns: An audit.
         :raises: :py:class:`~.AuditNotFound`
         """
 
     @abc.abstractmethod
-    def get_audit_by_uuid(self, context, audit_uuid):
+    def get_audit_by_uuid(self, context, audit_uuid, eager=False):
         """Return an audit.
 
         :param context: The security context
         :param audit_uuid: The uuid of an audit.
+        :param eager: If True, also loads One-to-X data (Default: False)
         :returns: An audit.
         :raises: :py:class:`~.AuditNotFound`
         """
@@ -402,29 +423,27 @@ class BaseConnection(object):
         """Soft delete an audit and all associated action plans.
 
         :param audit_id: The id or uuid of an audit.
-        :returns: An audit.
         :raises: :py:class:`~.AuditNotFound`
         """
 
     @abc.abstractmethod
-    def get_action_list(self, context, columns=None, filters=None, limit=None,
-                        marker=None, sort_key=None, sort_dir=None):
+    def get_action_list(self, context, filters=None, limit=None,
+                        marker=None, sort_key=None, sort_dir=None,
+                        eager=False):
         """Get specific columns for matching actions.
 
         Return a list of the specified columns for all actions that match the
         specified filters.
 
         :param context: The security context
-        :param columns: List of column names to return.
-                        Defaults to 'id' column when columns == None.
         :param filters: Filters to apply. Defaults to None.
-
         :param limit: Maximum number of actions to return.
         :param marker: the last item of the previous page; we return the next
                        result set.
         :param sort_key: Attribute by which results should be sorted.
         :param sort_dir: direction in which results should be sorted.
                          (asc, desc)
+        :param eager: If True, also loads One-to-X data (Default: False)
         :returns: A list of tuples of the specified columns.
         """
 
@@ -449,21 +468,23 @@ class BaseConnection(object):
         """
 
     @abc.abstractmethod
-    def get_action_by_id(self, context, action_id):
+    def get_action_by_id(self, context, action_id, eager=False):
         """Return a action.
 
         :param context: The security context
         :param action_id: The id of a action.
+        :param eager: If True, also loads One-to-X data (Default: False)
         :returns: A action.
         :raises: :py:class:`~.ActionNotFound`
         """
 
     @abc.abstractmethod
-    def get_action_by_uuid(self, context, action_uuid):
+    def get_action_by_uuid(self, context, action_uuid, eager=False):
         """Return a action.
 
         :param context: The security context
         :param action_uuid: The uuid of a action.
+        :param eager: If True, also loads One-to-X data (Default: False)
         :returns: A action.
         :raises: :py:class:`~.ActionNotFound`
         """
@@ -488,26 +509,31 @@ class BaseConnection(object):
         :raises: :py:class:`~.Invalid`
         """
 
+    def soft_delete_action(self, action_id):
+        """Soft delete an action.
+
+        :param action_id: The id or uuid of an action.
+        :raises: :py:class:`~.ActionNotFound`
+        """
+
     @abc.abstractmethod
     def get_action_plan_list(
-            self, context, columns=None, filters=None, limit=None,
-            marker=None, sort_key=None, sort_dir=None):
+            self, context, filters=None, limit=None,
+            marker=None, sort_key=None, sort_dir=None, eager=False):
         """Get specific columns for matching action plans.
 
         Return a list of the specified columns for all action plans that
         match the specified filters.
 
         :param context: The security context
-        :param columns: List of column names to return.
-                        Defaults to 'id' column when columns == None.
         :param filters: Filters to apply. Defaults to None.
-
         :param limit: Maximum number of audits to return.
         :param marker: the last item of the previous page; we return the next
                        result set.
         :param sort_key: Attribute by which results should be sorted.
         :param sort_dir: direction in which results should be sorted.
                          (asc, desc)
+        :param eager: If True, also loads One-to-X data (Default: False)
         :returns: A list of tuples of the specified columns.
         """
 
@@ -522,21 +548,23 @@ class BaseConnection(object):
         """
 
     @abc.abstractmethod
-    def get_action_plan_by_id(self, context, action_plan_id):
+    def get_action_plan_by_id(self, context, action_plan_id, eager=False):
         """Return an action plan.
 
         :param context: The security context
         :param action_plan_id: The id of an action plan.
+        :param eager: If True, also loads One-to-X data (Default: False)
         :returns: An action plan.
         :raises: :py:class:`~.ActionPlanNotFound`
         """
 
     @abc.abstractmethod
-    def get_action_plan_by_uuid(self, context, action_plan__uuid):
+    def get_action_plan_by_uuid(self, context, action_plan__uuid, eager=False):
         """Return a action plan.
 
         :param context: The security context
         :param action_plan__uuid: The uuid of an action plan.
+        :param eager: If True, also loads One-to-X data (Default: False)
         :returns: An action plan.
         :raises: :py:class:`~.ActionPlanNotFound`
         """
@@ -560,3 +588,284 @@ class BaseConnection(object):
         :raises: :py:class:`~.ActionPlanReferenced`
         :raises: :py:class:`~.Invalid`
         """
+
+    def soft_delete_action_plan(self, action_plan_id):
+        """Soft delete an action plan.
+
+        :param action_plan_id: The id or uuid of an action plan.
+        :raises: :py:class:`~.ActionPlanNotFound`
+        """
+
+    @abc.abstractmethod
+    def get_efficacy_indicator_list(self, context, filters=None, limit=None,
+                                    marker=None, sort_key=None, sort_dir=None,
+                                    eager=False):
+        """Get specific columns for matching efficacy indicators.
+
+        Return a list of the specified columns for all efficacy indicators that
+        match the specified filters.
+
+        :param context: The security context
+        :param columns: List of column names to return.
+                        Defaults to 'id' column when columns == None.
+        :param filters: Filters to apply. Defaults to None.
+
+        :param limit: Maximum number of efficacy indicators to return.
+        :param marker: The last item of the previous page; we return the next
+                       result set.
+        :param sort_key: Attribute by which results should be sorted.
+        :param sort_dir: Direction in which results should be sorted.
+                         (asc, desc)
+        :param eager: If True, also loads One-to-X data (Default: False)
+        :returns: A list of tuples of the specified columns.
+        """
+
+    @abc.abstractmethod
+    def create_efficacy_indicator(self, values):
+        """Create a new efficacy indicator.
+
+        :param values: A dict containing items used to identify
+                       and track the efficacy indicator. For example:
+
+                       ::
+
+                        {
+                         'id': 1,
+                         'uuid': utils.generate_uuid(),
+                         'name': 'my_efficacy_indicator',
+                         'display_name': 'My efficacy indicator',
+                         'goal_uuid': utils.generate_uuid(),
+                        }
+        :returns: An efficacy_indicator
+        :raises: :py:class:`~.EfficacyIndicatorAlreadyExists`
+        """
+
+    @abc.abstractmethod
+    def get_efficacy_indicator_by_id(self, context, efficacy_indicator_id,
+                                     eager=False):
+        """Return an efficacy indicator given its ID.
+
+        :param context: The security context
+        :param efficacy_indicator_id: The ID of an efficacy indicator
+        :param eager: If True, also loads One-to-X data (Default: False)
+        :returns: An efficacy indicator
+        :raises: :py:class:`~.EfficacyIndicatorNotFound`
+        """
+
+    @abc.abstractmethod
+    def get_efficacy_indicator_by_uuid(self, context, efficacy_indicator_uuid,
+                                       eager=False):
+        """Return an efficacy indicator given its UUID.
+
+        :param context: The security context
+        :param efficacy_indicator_uuid: The UUID of an efficacy indicator
+        :param eager: If True, also loads One-to-X data (Default: False)
+        :returns: An efficacy indicator
+        :raises: :py:class:`~.EfficacyIndicatorNotFound`
+        """
+
+    @abc.abstractmethod
+    def get_efficacy_indicator_by_name(self, context, efficacy_indicator_name,
+                                       eager=False):
+        """Return an efficacy indicator given its name.
+
+        :param context: The security context
+        :param efficacy_indicator_name: The name of an efficacy indicator
+        :param eager: If True, also loads One-to-X data (Default: False)
+        :returns: An efficacy indicator
+        :raises: :py:class:`~.EfficacyIndicatorNotFound`
+        """
+
+    @abc.abstractmethod
+    def destroy_efficacy_indicator(self, efficacy_indicator_uuid):
+        """Destroy an efficacy indicator.
+
+        :param efficacy_indicator_uuid: The UUID of an efficacy indicator
+        :raises: :py:class:`~.EfficacyIndicatorNotFound`
+        """
+
+    @abc.abstractmethod
+    def update_efficacy_indicator(self, efficacy_indicator_id, values):
+        """Update properties of an efficacy indicator.
+
+        :param efficacy_indicator_id: The ID of an efficacy indicator
+        :returns: An efficacy indicator
+        :raises: :py:class:`~.EfficacyIndicatorNotFound`
+        :raises: :py:class:`~.Invalid`
+        """
+
+    @abc.abstractmethod
+    def get_scoring_engine_list(
+            self, context, columns=None, filters=None, limit=None,
+            marker=None, sort_key=None, sort_dir=None, eager=False):
+        """Get specific columns for matching scoring engines.
+
+        Return a list of the specified columns for all scoring engines that
+        match the specified filters.
+
+        :param context: The security context
+        :param columns: List of column names to return.
+                        Defaults to 'id' column when columns == None.
+        :param filters: Filters to apply. Defaults to None.
+        :param limit: Maximum number of scoring engines to return.
+        :param marker: the last item of the previous page; we return the next
+                       result set.
+        :param sort_key: Attribute by which results should be sorted.
+        :param sort_dir: direction in which results should be sorted.
+                         (asc, desc)
+        :param eager: If True, also loads One-to-X data (Default: False)
+        :returns: A list of tuples of the specified columns.
+        """
+
+    @abc.abstractmethod
+    def create_scoring_engine(self, values):
+        """Create a new scoring engine.
+
+        :param values: A dict containing several items used to identify
+                       and track the scoring engine.
+        :returns: A scoring engine.
+        :raises: :py:class:`~.ScoringEngineAlreadyExists`
+        """
+
+    @abc.abstractmethod
+    def get_scoring_engine_by_id(self, context, scoring_engine_id,
+                                 eager=False):
+        """Return a scoring engine by its id.
+
+        :param context: The security context
+        :param scoring_engine_id: The id of a scoring engine.
+        :param eager: If True, also loads One-to-X data (Default: False)
+        :returns: A scoring engine.
+        :raises: :py:class:`~.ScoringEngineNotFound`
+        """
+
+    @abc.abstractmethod
+    def get_scoring_engine_by_uuid(self, context, scoring_engine_uuid,
+                                   eager=False):
+        """Return a scoring engine by its uuid.
+
+        :param context: The security context
+        :param scoring_engine_uuid: The uuid of a scoring engine.
+        :param eager: If True, also loads One-to-X data (Default: False)
+        :returns: A scoring engine.
+        :raises: :py:class:`~.ScoringEngineNotFound`
+        """
+
+    @abc.abstractmethod
+    def get_scoring_engine_by_name(self, context, scoring_engine_name,
+                                   eager=False):
+        """Return a scoring engine by its name.
+
+        :param context: The security context
+        :param scoring_engine_name: The name of a scoring engine.
+        :param eager: If True, also loads One-to-X data (Default: False)
+        :returns: A scoring engine.
+        :raises: :py:class:`~.ScoringEngineNotFound`
+        """
+
+    @abc.abstractmethod
+    def destroy_scoring_engine(self, scoring_engine_id):
+        """Destroy a scoring engine.
+
+        :param scoring_engine_id: The id of a scoring engine.
+        :raises: :py:class:`~.ScoringEngineNotFound`
+        """
+
+    @abc.abstractmethod
+    def update_scoring_engine(self, scoring_engine_id, values):
+        """Update properties of a scoring engine.
+
+        :param scoring_engine_id: The id of a scoring engine.
+        :returns: A scoring engine.
+        :raises: :py:class:`~.ScoringEngineNotFound`
+        :raises: :py:class:`~.Invalid`
+        """
+
+    @abc.abstractmethod
+    def get_service_list(self, context, filters=None, limit=None, marker=None,
+                         sort_key=None, sort_dir=None, eager=False):
+        """Get specific columns for matching services.
+
+        Return a list of the specified columns for all services that
+        match the specified filters.
+
+        :param context: The security context
+        :param filters: Filters to apply. Defaults to None.
+
+        :param limit: Maximum number of services to return.
+        :param marker: The last item of the previous page; we return the next
+                       result set.
+        :param sort_key: Attribute by which results should be sorted.
+        :param sort_dir: Direction in which results should be sorted.
+                         (asc, desc)
+        :param eager: If True, also loads One-to-X data (Default: False)
+        :returns: A list of tuples of the specified columns.
+        """
+
+    @abc.abstractmethod
+    def create_service(self, values):
+        """Create a new service.
+
+        :param values: A dict containing items used to identify
+                       and track the service. For example:
+
+                       ::
+
+                        {
+                         'id': 1,
+                         'name': 'watcher-api',
+                         'status': 'ACTIVE',
+                         'host': 'controller'
+                        }
+        :returns: A service
+        :raises: :py:class:`~.ServiceAlreadyExists`
+        """
+
+    @abc.abstractmethod
+    def get_service_by_id(self, context, service_id, eager=False):
+        """Return a service given its ID.
+
+        :param context: The security context
+        :param service_id: The ID of a service
+        :param eager: If True, also loads One-to-X data (Default: False)
+        :returns: A service
+        :raises: :py:class:`~.ServiceNotFound`
+        """
+
+    @abc.abstractmethod
+    def get_service_by_name(self, context, service_name, eager=False):
+        """Return a service given its name.
+
+        :param context: The security context
+        :param service_name: The name of a service
+        :param eager: If True, also loads One-to-X data (Default: False)
+        :returns: A service
+        :raises: :py:class:`~.ServiceNotFound`
+        """
+
+    @abc.abstractmethod
+    def destroy_service(self, service_id):
+        """Destroy a service.
+
+        :param service_id: The ID of a service
+        :raises: :py:class:`~.ServiceNotFound`
+        """
+
+    @abc.abstractmethod
+    def update_service(self, service_id, values):
+        """Update properties of a service.
+
+        :param service_id: The ID of a service
+        :returns: A service
+        :raises: :py:class:`~.ServiceyNotFound`
+        :raises: :py:class:`~.Invalid`
+        """
+
+    @abc.abstractmethod
+    def soft_delete_service(self, service_id):
+        """Soft delete a service.
+
+        :param service_id: The id of a service.
+        :returns: A service.
+        :raises: :py:class:`~.ServiceNotFound`
+        """
