@@ -42,12 +42,15 @@ airflow is higher than the specified threshold.
 - It assumes that live migrations are possible.
 """
 
+import datetime
 
+from oslo_config import cfg
 from oslo_log import log
 
-from watcher._i18n import _, _LE, _LI, _LW
+from watcher._i18n import _
 from watcher.common import exception as wexc
-from watcher.decision_engine.cluster.history import ceilometer as ceil
+from watcher.datasource import ceilometer as ceil
+from watcher.datasource import gnocchi as gnoc
 from watcher.decision_engine.model import element
 from watcher.decision_engine.strategy.strategies import base
 
@@ -80,15 +83,28 @@ class UniformAirflow(base.BaseStrategy):
        - It assumes that live migrations are possible.
     """
 
-    # The meter to report Airflow of physical server in ceilometer
-    METER_NAME_AIRFLOW = "hardware.ipmi.node.airflow"
-    # The meter to report inlet temperature of physical server in ceilometer
-    METER_NAME_INLET_T = "hardware.ipmi.node.temperature"
-    # The meter to report system power of physical server in ceilometer
-    METER_NAME_POWER = "hardware.ipmi.node.power"
     # choose 300 seconds as the default duration of meter aggregation
     PERIOD = 300
 
+    METRIC_NAMES = dict(
+        ceilometer=dict(
+            # The meter to report Airflow of physical server in ceilometer
+            host_airflow='hardware.ipmi.node.airflow',
+            # The meter to report inlet temperature of physical server
+            # in ceilometer
+            host_inlet_temp='hardware.ipmi.node.temperature',
+            # The meter to report system power of physical server in ceilometer
+            host_power='hardware.ipmi.node.power'),
+        gnocchi=dict(
+            # The meter to report Airflow of physical server in gnocchi
+            host_airflow='hardware.ipmi.node.airflow',
+            # The meter to report inlet temperature of physical server
+            # in gnocchi
+            host_inlet_temp='hardware.ipmi.node.temperature',
+            # The meter to report system power of physical server in gnocchi
+            host_power='hardware.ipmi.node.power'),
+    )
+
     MIGRATION = "migrate"
 
     def __init__(self, config, osc=None):
@@ -101,22 +117,36 @@ class UniformAirflow(base.BaseStrategy):
         super(UniformAirflow, self).__init__(config, osc)
         # The migration plan will be triggered when the airflow reaches
         # threshold
-        self.meter_name_airflow = self.METER_NAME_AIRFLOW
-        self.meter_name_inlet_t = self.METER_NAME_INLET_T
-        self.meter_name_power = self.METER_NAME_POWER
+        self.meter_name_airflow = self.METRIC_NAMES[
+            self.config.datasource]['host_airflow']
+        self.meter_name_inlet_t = self.METRIC_NAMES[
+            self.config.datasource]['host_inlet_temp']
+        self.meter_name_power = self.METRIC_NAMES[
+            self.config.datasource]['host_power']
         self._ceilometer = None
+        self._gnocchi = None
         self._period = self.PERIOD
 
     @property
     def ceilometer(self):
         if self._ceilometer is None:
-            self._ceilometer = ceil.CeilometerClusterHistory(osc=self.osc)
+            self._ceilometer = ceil.CeilometerHelper(osc=self.osc)
         return self._ceilometer
 
     @ceilometer.setter
     def ceilometer(self, c):
         self._ceilometer = c
 
+    @property
+    def gnocchi(self):
+        if self._gnocchi is None:
+            self._gnocchi = gnoc.GnocchiHelper(osc=self.osc)
+        return self._gnocchi
+
+    @gnocchi.setter
+    def gnocchi(self, g):
+        self._gnocchi = g
+
     @classmethod
     def get_name(cls):
         return "uniform_airflow"
@@ -133,6 +163,10 @@ class UniformAirflow(base.BaseStrategy):
     def get_goal_name(cls):
         return "airflow_optimization"
 
+    @property
+    def granularity(self):
+        return self.input_parameters.get('granularity', 300)
+
     @classmethod
     def get_schema(cls):
         # Mandatory default setting for each element
@@ -161,21 +195,35 @@ class UniformAirflow(base.BaseStrategy):
                     "type": "number",
                     "default": 300
                 },
+                "granularity": {
+                    "description": "The time between two measures in an "
+                                   "aggregated timeseries of a metric.",
+                    "type": "number",
+                    "default": 300
+                },
             },
         }
 
-    def calculate_used_resource(self, node, cap_cores, cap_mem, cap_disk):
+    @classmethod
+    def get_config_opts(cls):
+        return [
+            cfg.StrOpt(
+                "datasource",
+                help="Data source to use in order to query the needed metrics",
+                default="ceilometer",
+                choices=["ceilometer", "gnocchi"])
+        ]
+
+    def calculate_used_resource(self, node):
         """Compute the used vcpus, memory and disk based on instance flavors"""
-        instances = self.compute_model.mapping.get_node_instances(node)
+        instances = self.compute_model.get_node_instances(node)
         vcpus_used = 0
         memory_mb_used = 0
         disk_gb_used = 0
-        for instance_id in instances:
-            instance = self.compute_model.get_instance_by_uuid(
-                instance_id)
-            vcpus_used += cap_cores.get_capacity(instance)
-            memory_mb_used += cap_mem.get_capacity(instance)
-            disk_gb_used += cap_disk.get_capacity(instance)
+        for instance in instances:
+            vcpus_used += instance.vcpus
+            memory_mb_used += instance.memory
+            disk_gb_used += instance.disk
 
         return vcpus_used, memory_mb_used, disk_gb_used
 
@@ -187,71 +235,71 @@ class UniformAirflow(base.BaseStrategy):
         instances_tobe_migrate = []
         for nodemap in hosts:
             source_node = nodemap['node']
-            source_instances = self.compute_model.mapping.get_node_instances(
+            source_instances = self.compute_model.get_node_instances(
                 source_node)
             if source_instances:
-                inlet_t = self.ceilometer.statistic_aggregation(
-                    resource_id=source_node.uuid,
-                    meter_name=self.meter_name_inlet_t,
-                    period=self._period,
-                    aggregate='avg')
-                power = self.ceilometer.statistic_aggregation(
-                    resource_id=source_node.uuid,
-                    meter_name=self.meter_name_power,
-                    period=self._period,
-                    aggregate='avg')
+                if self.config.datasource == "ceilometer":
+                    inlet_t = self.ceilometer.statistic_aggregation(
+                        resource_id=source_node.uuid,
+                        meter_name=self.meter_name_inlet_t,
+                        period=self._period,
+                        aggregate='avg')
+                    power = self.ceilometer.statistic_aggregation(
+                        resource_id=source_node.uuid,
+                        meter_name=self.meter_name_power,
+                        period=self._period,
+                        aggregate='avg')
+                elif self.config.datasource == "gnocchi":
+                    stop_time = datetime.datetime.utcnow()
+                    start_time = stop_time - datetime.timedelta(
+                        seconds=int(self._period))
+                    inlet_t = self.gnocchi.statistic_aggregation(
+                        resource_id=source_node.uuid,
+                        metric=self.meter_name_inlet_t,
+                        granularity=self.granularity,
+                        start_time=start_time,
+                        stop_time=stop_time,
+                        aggregation='mean')
+                    power = self.gnocchi.statistic_aggregation(
+                        resource_id=source_node.uuid,
+                        metric=self.meter_name_power,
+                        granularity=self.granularity,
+                        start_time=start_time,
+                        stop_time=stop_time,
+                        aggregation='mean')
                 if (power < self.threshold_power and
                         inlet_t < self.threshold_inlet_t):
                     # hardware issue, migrate all instances from this node
-                    for instance_id in source_instances:
-                        try:
-                            instance = (self.compute_model.
-                                        get_instance_by_uuid(instance_id))
-                            instances_tobe_migrate.append(instance)
-                        except wexc.InstanceNotFound:
-                            LOG.error(_LE("Instance not found; error: %s"),
-                                      instance_id)
+                    for instance in source_instances:
+                        instances_tobe_migrate.append(instance)
                     return source_node, instances_tobe_migrate
                 else:
                     # migrate the first active instance
-                    for instance_id in source_instances:
-                        try:
-                            instance = (self.compute_model.
-                                        get_instance_by_uuid(instance_id))
-                            if (instance.state !=
-                                    element.InstanceState.ACTIVE.value):
-                                LOG.info(
-                                    _LI("Instance not active, skipped: %s"),
-                                    instance.uuid)
-                                continue
-                            instances_tobe_migrate.append(instance)
-                            return source_node, instances_tobe_migrate
-                        except wexc.InstanceNotFound:
-                            LOG.error(_LE("Instance not found; error: %s"),
-                                      instance_id)
+                    for instance in source_instances:
+                        if (instance.state !=
+                                element.InstanceState.ACTIVE.value):
+                            LOG.info(
+                                "Instance not active, skipped: %s",
+                                instance.uuid)
+                            continue
+                        instances_tobe_migrate.append(instance)
+                        return source_node, instances_tobe_migrate
             else:
-                LOG.info(_LI("Instance not found on node: %s"),
+                LOG.info("Instance not found on node: %s",
                          source_node.uuid)
 
     def filter_destination_hosts(self, hosts, instances_to_migrate):
         """Find instance and host with sufficient available resources"""
-
-        cap_cores = self.compute_model.get_resource_by_uuid(
-            element.ResourceType.cpu_cores)
-        cap_disk = self.compute_model.get_resource_by_uuid(
-            element.ResourceType.disk)
-        cap_mem = self.compute_model.get_resource_by_uuid(
-            element.ResourceType.memory)
-        # large instance go first
+        # large instances go first
         instances_to_migrate = sorted(
             instances_to_migrate, reverse=True,
-            key=lambda x: (cap_cores.get_capacity(x)))
+            key=lambda x: (x.vcpus))
         # find hosts for instances
         destination_hosts = []
         for instance_to_migrate in instances_to_migrate:
-            required_cores = cap_cores.get_capacity(instance_to_migrate)
-            required_disk = cap_disk.get_capacity(instance_to_migrate)
-            required_mem = cap_mem.get_capacity(instance_to_migrate)
+            required_cores = instance_to_migrate.vcpus
+            required_disk = instance_to_migrate.disk
+            required_mem = instance_to_migrate.memory
             dest_migrate_info = {}
             for nodemap in hosts:
                 host = nodemap['node']
@@ -259,13 +307,13 @@ class UniformAirflow(base.BaseStrategy):
                     # calculate the available resources
                     nodemap['cores_used'], nodemap['mem_used'],\
                         nodemap['disk_used'] = self.calculate_used_resource(
-                            host, cap_cores, cap_mem, cap_disk)
-                cores_available = (cap_cores.get_capacity(host) -
+                            host)
+                cores_available = (host.vcpus -
                                    nodemap['cores_used'])
-                disk_available = (cap_disk.get_capacity(host) -
+                disk_available = (host.disk -
                                   nodemap['disk_used'])
                 mem_available = (
-                    cap_mem.get_capacity(host) - nodemap['mem_used'])
+                    host.memory - nodemap['mem_used'])
                 if (cores_available >= required_cores and
                         disk_available >= required_disk and
                         mem_available >= required_mem):
@@ -278,8 +326,8 @@ class UniformAirflow(base.BaseStrategy):
                     break
         # check if all instances have target hosts
         if len(destination_hosts) != len(instances_to_migrate):
-            LOG.warning(_LW("Not all target hosts could be found; it might "
-                            "be because there is not enough resource"))
+            LOG.warning("Not all target hosts could be found; it might "
+                        "be because there is not enough resource")
             return None
         return destination_hosts
 
@@ -292,17 +340,30 @@ class UniformAirflow(base.BaseStrategy):
         overload_hosts = []
         nonoverload_hosts = []
         for node_id in nodes:
+            airflow = None
             node = self.compute_model.get_node_by_uuid(
                 node_id)
             resource_id = node.uuid
-            airflow = self.ceilometer.statistic_aggregation(
-                resource_id=resource_id,
-                meter_name=self.meter_name_airflow,
-                period=self._period,
-                aggregate='avg')
+            if self.config.datasource == "ceilometer":
+                airflow = self.ceilometer.statistic_aggregation(
+                    resource_id=resource_id,
+                    meter_name=self.meter_name_airflow,
+                    period=self._period,
+                    aggregate='avg')
+            elif self.config.datasource == "gnocchi":
+                stop_time = datetime.datetime.utcnow()
+                start_time = stop_time - datetime.timedelta(
+                    seconds=int(self._period))
+                airflow = self.gnocchi.statistic_aggregation(
+                    resource_id=resource_id,
+                    metric=self.meter_name_airflow,
+                    granularity=self.granularity,
+                    start_time=start_time,
+                    stop_time=stop_time,
+                    aggregation='mean')
             # some hosts may not have airflow meter, remove from target
             if airflow is None:
-                LOG.warning(_LW("%s: no airflow data"), resource_id)
+                LOG.warning("%s: no airflow data", resource_id)
                 continue
 
             LOG.debug("%s: airflow %f" % (resource_id, airflow))
@@ -337,9 +398,9 @@ class UniformAirflow(base.BaseStrategy):
             return self.solution
 
         if not target_nodes:
-            LOG.warning(_LW("No hosts currently have airflow under %s, "
-                            "therefore there are no possible target "
-                            "hosts for any migration"),
+            LOG.warning("No hosts currently have airflow under %s, "
+                        "therefore there are no possible target "
+                        "hosts for any migration",
                         self.threshold_airflow)
             return self.solution
 
@@ -358,8 +419,8 @@ class UniformAirflow(base.BaseStrategy):
         destination_hosts = self.filter_destination_hosts(
             target_nodes, instances_src)
         if not destination_hosts:
-            LOG.warning(_LW("No target host could be found; it might "
-                            "be because there is not enough resources"))
+            LOG.warning("No target host could be found; it might "
+                        "be because there is not enough resources")
             return self.solution
         # generate solution to migrate the instance to the dest server,
         for info in destination_hosts:
