@@ -16,8 +16,11 @@
 
 from __future__ import unicode_literals
 
+import functools
+
+from tempest.lib.common.utils import test_utils
+from tempest.lib import decorators
 from tempest.lib import exceptions
-from tempest import test
 
 from watcher_tempest_plugin.tests.api.admin import base
 
@@ -25,8 +28,8 @@ from watcher_tempest_plugin.tests.api.admin import base
 class TestCreateUpdateDeleteAudit(base.BaseInfraOptimTest):
     """Tests for audit."""
 
-    audit_states = ['ONGOING', 'SUCCEEDED', 'SUBMITTED', 'FAILED',
-                    'CANCELLED', 'DELETED', 'PENDING']
+    audit_states = ['ONGOING', 'SUCCEEDED', 'FAILED',
+                    'CANCELLED', 'DELETED', 'PENDING', 'SUSPENDED']
 
     def assert_expected(self, expected, actual,
                         keys=('created_at', 'updated_at',
@@ -34,51 +37,56 @@ class TestCreateUpdateDeleteAudit(base.BaseInfraOptimTest):
         super(TestCreateUpdateDeleteAudit, self).assert_expected(
             expected, actual, keys)
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_create_audit_oneshot(self):
-        _, goal = self.client.show_goal("DUMMY")
+        _, goal = self.client.show_goal("dummy")
         _, audit_template = self.create_audit_template(goal['uuid'])
 
         audit_params = dict(
             audit_template_uuid=audit_template['uuid'],
-            type='ONESHOT',
+            audit_type='ONESHOT',
         )
 
         _, body = self.create_audit(**audit_params)
+        audit_params.pop('audit_template_uuid')
+        audit_params['goal_uuid'] = goal['uuid']
         self.assert_expected(audit_params, body)
 
         _, audit = self.client.show_audit(body['uuid'])
         self.assert_expected(audit, body)
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_create_audit_continuous(self):
-        _, goal = self.client.show_goal("DUMMY")
+        _, goal = self.client.show_goal("dummy")
         _, audit_template = self.create_audit_template(goal['uuid'])
 
         audit_params = dict(
             audit_template_uuid=audit_template['uuid'],
-            type='CONTINUOUS',
+            audit_type='CONTINUOUS',
+            interval='7200',
         )
 
         _, body = self.create_audit(**audit_params)
+        audit_params.pop('audit_template_uuid')
+        audit_params['goal_uuid'] = goal['uuid']
         self.assert_expected(audit_params, body)
 
         _, audit = self.client.show_audit(body['uuid'])
         self.assert_expected(audit, body)
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_create_audit_with_wrong_audit_template(self):
         audit_params = dict(
             audit_template_uuid='INVALID',
-            type='ONESHOT',
+            audit_type='ONESHOT',
         )
 
         self.assertRaises(
             exceptions.BadRequest, self.create_audit, **audit_params)
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_create_audit_with_invalid_state(self):
-        _, goal = self.client.show_goal("DUMMY")
+        _, goal = self.client.show_goal("dummy")
         _, audit_template = self.create_audit_template(goal['uuid'])
 
         audit_params = dict(
@@ -89,9 +97,9 @@ class TestCreateUpdateDeleteAudit(base.BaseInfraOptimTest):
         self.assertRaises(
             exceptions.BadRequest, self.create_audit, **audit_params)
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_create_audit_with_no_state(self):
-        _, goal = self.client.show_goal("DUMMY")
+        _, goal = self.client.show_goal("dummy")
         _, audit_template = self.create_audit_template(goal['uuid'])
 
         audit_params = dict(
@@ -100,6 +108,8 @@ class TestCreateUpdateDeleteAudit(base.BaseInfraOptimTest):
         )
 
         _, body = self.create_audit(**audit_params)
+        audit_params.pop('audit_template_uuid')
+        audit_params['goal_uuid'] = goal['uuid']
         self.assert_expected(audit_params, body)
 
         _, audit = self.client.show_audit(body['uuid'])
@@ -109,29 +119,47 @@ class TestCreateUpdateDeleteAudit(base.BaseInfraOptimTest):
 
         self.assert_expected(audit, body)
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_delete_audit(self):
-        _, goal = self.client.show_goal("DUMMY")
+        _, goal = self.client.show_goal("dummy")
         _, audit_template = self.create_audit_template(goal['uuid'])
         _, body = self.create_audit(audit_template['uuid'])
         audit_uuid = body['uuid']
 
+        test_utils.call_until_true(
+            func=functools.partial(
+                self.is_audit_idle, audit_uuid),
+            duration=10,
+            sleep_for=.5
+        )
+
+        def is_audit_deleted(uuid):
+            try:
+                return not bool(self.client.show_audit(uuid))
+            except exceptions.NotFound:
+                return True
+
         self.delete_audit(audit_uuid)
 
-        self.assertRaises(
-            exceptions.NotFound, self.client.show_audit, audit_uuid)
+        test_utils.call_until_true(
+            func=functools.partial(is_audit_deleted, audit_uuid),
+            duration=5,
+            sleep_for=1
+        )
+
+        self.assertTrue(is_audit_deleted(audit_uuid))
 
 
 class TestShowListAudit(base.BaseInfraOptimTest):
     """Tests for audit."""
 
-    audit_states = ['ONGOING', 'SUCCEEDED', 'SUBMITTED', 'FAILED',
-                    'CANCELLED', 'DELETED', 'PENDING']
+    audit_states = ['ONGOING', 'SUCCEEDED', 'FAILED',
+                    'CANCELLED', 'DELETED', 'PENDING', 'SUSPENDED']
 
     @classmethod
     def resource_setup(cls):
         super(TestShowListAudit, cls).resource_setup()
-        _, cls.goal = cls.client.show_goal("DUMMY")
+        _, cls.goal = cls.client.show_goal("dummy")
         _, cls.audit_template = cls.create_audit_template(cls.goal['uuid'])
         _, cls.audit = cls.create_audit(cls.audit_template['uuid'])
 
@@ -141,7 +169,7 @@ class TestShowListAudit(base.BaseInfraOptimTest):
         super(TestShowListAudit, self).assert_expected(
             expected, actual, keys)
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_show_audit(self):
         _, audit = self.client.show_audit(
             self.audit['uuid'])
@@ -155,7 +183,7 @@ class TestShowListAudit(base.BaseInfraOptimTest):
         self.assertIn(audit_state, self.audit_states)
         self.assert_expected(initial_audit, actual_audit)
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_show_audit_with_links(self):
         _, audit = self.client.show_audit(
             self.audit['uuid'])
@@ -164,7 +192,7 @@ class TestShowListAudit(base.BaseInfraOptimTest):
         self.assertIn(audit['uuid'],
                       audit['links'][0]['href'])
 
-    @test.attr(type="smoke")
+    @decorators.attr(type="smoke")
     def test_list_audits(self):
         _, body = self.client.list_audits()
         self.assertIn(self.audit['uuid'],
@@ -174,7 +202,7 @@ class TestShowListAudit(base.BaseInfraOptimTest):
             self.validate_self_link('audits', audit['uuid'],
                                     audit['links'][0]['href'])
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_list_with_limit(self):
         # We create 3 extra audits to exceed the limit we fix
         for _ in range(3):
@@ -186,8 +214,8 @@ class TestShowListAudit(base.BaseInfraOptimTest):
         self.assertEqual(3, len(body['audits']))
         self.assertIn(next_marker, body['next'])
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_list_audits_related_to_given_audit_template(self):
         _, body = self.client.list_audits(
-            audit_template=self.audit_template['uuid'])
+            goal=self.goal['uuid'])
         self.assertIn(self.audit['uuid'], [n['uuid'] for n in body['audits']])
